library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity SD_Card_IF is
port(
	-- host side signals (Avalon Bus)
	signal CLK		: in std_logic;
	signal iCMD		: in std_logic;
	signal iCS_n	: in std_logic;
	signal iRST_n	: in std_logic;
	signal iIORD_n	: in std_logic;
	signal iIOWR_n	: in std_logic;
	
	signal iDATA	: in std_logic_vector(7 downto 0);
	signal oDATA	: out std_logic_vector(7 downto 0);

	-- peripheral side signals (SD card)
	signal SD_DET	: in std_logic;		-- SD card detect signal
	signal SD_MISO	: in std_logic;		-- master (FPGA) in, slave (SD-Card)out
	signal SD_MOSI	: out std_logic;	-- master out, slave in
	signal SD_CS_n	: out std_logic;	-- chip select
	signal SD_CLK	: inout std_logic);	-- clock
end entity SD_Card_IF;

architecture behaviour of SD_Card_IF is
	signal cmd_reg: std_logic_vector(4 downto 0);		-- command register
														-- bit 0 -> SD card detection (0-no SD card, 1-SD card inserted)
														-- bit 1 -> sd card data clock selection (0=400kHz, 1=25MHz)
														-- bit 2 -> tx/rx ready (0-data receiption/transmission finished, 1-data receiption/transmission in progress)
														-- bit 3 -> SD card IF reset (write '1' to reset IF)
														-- bit 4 -> SD Card chip select (0-SD card not selected, 1-SD card selected) 
	signal clk_div: std_logic_vector(6 downto 0);		-- clock divider factor
	signal bit_cntr: std_logic_vector(3 downto 0);		-- bit counter
	signal data_in_reg: std_logic_vector(7 downto 0);	-- input data shift register
	signal data_out_reg: std_logic_vector(7 downto 0);	-- output data shift register
begin
	cmd_reg(0) <= SD_DET;
	SD_CS_n <= not(cmd_reg(4));
	SD_MOSI <= data_out_reg(7);
	oDATA <= data_in_reg when iCMD='0' else "000"&cmd_reg;

	process(CLK, iRST_n)
	begin
		if CLK'event and CLK='1'  then
			if iRST_n = '0' or cmd_reg(3)='1' then	-- reset
				SD_CLK <= '0';
				bit_cntr <= "1000";
				clk_div <= (others=>'0');
				data_in_reg <= (others => '0');
				data_out_reg <= (others => '0');
				cmd_reg(4 downto 1) <= (others=>'0');
			else
				-- write CMD/DATA register
				if iCS_n='0' and iIOWR_n='0' then
					if iCMD='0' then
						data_out_reg <= iDATA;
						cmd_reg(2) <= '1';
					else
						cmd_reg(4 downto 1) <= iDATA(4 downto 1);
					end if;
				else
					-- SD card data/clock generation
					if cmd_reg(2)='0' or clk_div = "0000000" then
						-- reload clock divider at reset or transmission end
						if cmd_reg(1)='0' then
							clk_div <= "1111100"; 	-- div = 250/2-1
						else
							clk_div <= "0000001"; 	-- div = 4/2-1
						end if;
					end if;
				
					if cmd_reg(2)='0' then
						SD_CLK <= '0';
						bit_cntr <= "1000";
					elsif clk_div = "0000000" then
						SD_CLK <= not(SD_CLK);
						-- data receiption/transmission
						if SD_CLK='1' then							-- falling edge
							data_out_reg <= data_out_reg(6 downto 0) & '0';
						elsif SD_CLK='0' and bit_cntr /= "0000" then	-- rising edge
							bit_cntr <= bit_cntr - "0001";
							data_in_reg <= data_in_reg(6 downto 0) & SD_MISO;
						else
							SD_CLK <= '0';
							cmd_reg(2) <= '0';		-- end of receiption/transmission
						end if;
					else
						clk_div <= clk_div - "0000001";
					end if;
				end if;
			end if;
		end if;
	end process;
end behaviour;