library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity nios2_uclinux is 
port(
	-- global signals
	signal sys_clk	: in std_logic;
	signal sdram_clk: out std_logic;

	-- user LEDs
	signal ULEDs	: out std_logic_vector(2 downto 0);

	-- sdram memory interface
	signal sd_CS_n	: out std_logic_vector(1 downto 0);
	signal CAS_n	: out std_logic;
	signal RAS_n	: out std_logic;
	signal CKE0_n	: out std_logic;
	signal CKE1_n	: out std_logic;	
	signal SD_WE_n	: out std_logic;
	signal ba_addr	: out std_logic_vector (1 downto 0);
	signal sd_dqm	: out std_logic_vector (7 downto 0);
	signal sd_addr	: out std_logic_vector (12 downto 0);
	signal sd_data	: inout std_logic_vector (63 downto 0);

	-- flash memory interface
	signal BYTE_n		: out std_logic;
	signal RST_n		: out std_logic;
	signal CS_n			: out std_logic;
	signal OE_n			: out std_logic;
	signal WE_n			: out std_logic;
	signal flash_addr	: out std_logic_vector (23 downto 0);
	signal flash_data	: inout std_logic_vector (7 downto 0);

	-- SD-Card
	signal sd_det	: in std_logic;
	signal sd_clk	: inout std_logic;
	signal sd_cmd	: out std_logic;
	signal sd_d3	: inout std_logic;
	signal sd_d0	: inout std_logic;

	-- Davicom DM9000A interface
	signal eth_INT	: in std_logic;
	signal eth_CMD	: out std_logic;
	signal eth_CS_n	: out std_logic;
	signal eth_RD_n	: out std_logic;
	signal eth_WR_n	: out std_logic;
	signal eth_RST_n: out std_logic;
	signal eth_data	: inout std_logic_vector(15 downto 0));
end entity nios2_uclinux;

architecture structural of nios2_uclinux is

	component pll is
	port(
		inclk0	: in std_logic := '0';
		c0		: out std_logic;
		c2		: out std_logic;
		locked	: out std_logic);
	end component;
	
	component nios2_sys is 
	port (
		-- global signals:
		signal clk : IN STD_LOGIC;
		signal reset_n : IN STD_LOGIC;

		-- the_dm9000a
		signal CMD_from_the_dm9000a : OUT STD_LOGIC;
		signal CS_n_from_the_dm9000a : OUT STD_LOGIC;
		signal INT_to_the_dm9000a : IN STD_LOGIC;
		signal IORD_n_from_the_dm9000a : OUT STD_LOGIC;
		signal IOWR_n_to_and_from_the_DM9000A : OUT STD_LOGIC;
		signal RST_n_from_the_dm9000a : OUT STD_LOGIC;
		signal data_to_and_from_the_dm9000a : INOUT STD_LOGIC_VECTOR (15 DOWNTO 0);
		
		-- the_sd_card
		signal SD_CLK_to_and_from_the_sd_card : INOUT STD_LOGIC;
		signal SD_CS_n_from_the_sd_card : OUT STD_LOGIC;
		signal SD_DET_to_the_sd_card : IN STD_LOGIC;
		signal SD_MISO_to_the_sd_card : IN STD_LOGIC;
		signal SD_MOSI_from_the_sd_card : OUT STD_LOGIC;

		-- the_sdram
		signal zs_addr_from_the_sdram : OUT STD_LOGIC_VECTOR (12 DOWNTO 0);
		signal zs_ba_from_the_sdram : OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
		signal zs_cas_n_from_the_sdram : OUT STD_LOGIC;
		signal zs_cke_from_the_sdram : OUT STD_LOGIC;
		signal zs_cs_n_from_the_sdram : OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
		signal zs_dq_to_and_from_the_sdram : INOUT STD_LOGIC_VECTOR (63 DOWNTO 0);
		signal zs_dqm_from_the_sdram : OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
		signal zs_ras_n_from_the_sdram : OUT STD_LOGIC;
		signal zs_we_n_from_the_sdram : OUT STD_LOGIC;

		-- the_tri_state_bridge_avalon_slave
		signal address_to_the_flash : OUT STD_LOGIC_VECTOR (23 DOWNTO 0);
		signal read_n_to_the_flash : OUT STD_LOGIC;
		signal select_n_to_the_flash : OUT STD_LOGIC;
		signal tristate_bridge_data : INOUT STD_LOGIC_VECTOR (7 DOWNTO 0);
		signal write_n_to_the_flash : OUT STD_LOGIC);
	end component;

	signal clk, reset_n, cke_n : std_logic;
begin
	RST_n <= '1';
	BYTE_n <= '0';										-- 8 bit access to flash
	CKE0_n <= cke_n;
	CKE1_n <= cke_n;

	ULEDs(0) <= sd_d3;
	ULEDs(1) <= sd_clk;
	ULEDs(2) <= sd_d0;

	pll_inst : pll
	port map(
		inclk0	=> sys_clk,
		c0		=> clk,		
		c2		=> sdram_clk,
		locked	=> reset_n);

	system : nios2_sys
	port map(
		-- global signals:
		clk		=> clk,
		reset_n => reset_n,

		-- the_dm9000a
		CMD_from_the_dm9000a => eth_CMD,
		CS_n_from_the_dm9000a => eth_CS_n,
		INT_to_the_dm9000a => eth_INT,
		IORD_n_from_the_dm9000a => eth_RD_n,
		IOWR_n_to_and_from_the_DM9000A => eth_WR_n,
		RST_n_from_the_dm9000a => eth_RST_n,
		data_to_and_from_the_dm9000a => eth_data,

		-- SD Card
		SD_CLK_to_and_from_the_sd_card	=> sd_clk,
		SD_CS_n_from_the_sd_card	=> sd_d3,
		SD_DET_to_the_sd_card		=> sd_det,
		SD_MISO_to_the_sd_card		=> sd_d0,
		SD_MOSI_from_the_sd_card	=> sd_cmd,

		-- the_sdram
		zs_addr_from_the_sdram		=> sd_addr,
		zs_ba_from_the_sdram		=> ba_addr,
		zs_cas_n_from_the_sdram		=> CAS_n,
		zs_cke_from_the_sdram		=> cke_n,
		zs_cs_n_from_the_sdram		=> sd_CS_n,
		zs_dq_to_and_from_the_sdram	=> sd_data,
		zs_dqm_from_the_sdram		=> sd_dqm,
		zs_ras_n_from_the_sdram		=> RAS_n,
		zs_we_n_from_the_sdram		=> SD_WE_n,

		-- the_tri_state_bridge_0_avalon_slave
		address_to_the_flash		=> flash_addr,
		read_n_to_the_flash			=> OE_n,
		select_n_to_the_flash		=> CS_n,
		tristate_bridge_data		=> flash_data,
		write_n_to_the_flash		=> WE_n);
end structural;